package com.tripod.squadron.rogue;

import java.util.Arrays;
import java.util.Timer;
import java.util.TimerTask;

import android.app.Activity;
import android.os.Bundle;
import android.text.Html;
import android.view.View;
import android.widget.ImageButton;
import android.widget.TextView;

public class ANDIROiD_1_1Activity extends Activity {
	int playerClicks;
	int playerTime;
	private int aField[] = new int[16];
	private int theField[] = new int[16];
	Timer timer;
	
	// Generating a new game
	private void makeField () {
		int sel1, sel2, btnNo;
		aField = theField.clone();
		// Shuffle the puzzle field, even number of flips guarantees solubility
		for (int z=1; z<=28; z++) {
			sel1 = (int) Math.round(Math.random()*13);
			sel2 = sel1+1;
			btnNo = aField[sel2];
			aField[sel2] = aField[sel1];
			aField[sel1] = btnNo;
		}
		playerClicks = 0;
	}
	
	// Loading the puzzle field to the buttons
	private void loadField () {
		for (int z=0; z<=15; z++){
			int id = getResources().getIdentifier("btn"+z, "id", getPackageName());
			ImageButton btn = (ImageButton) findViewById(id);
			id = getResources().getIdentifier("cell"+ aField[z], "drawable", getPackageName());
			btn.setImageResource(id);
		}
	}

	// Getting an index in the puzzle field by row and column
	private int getIndex(int row, int col){
	  return row*4 + col;
	};
	
	// Getting the index of neighbour empty sell
	private int idxEmpty(int row, int col){
	  int theIdx = getIndex(row, col);
	  if (row>0)
	    if (aField[theIdx-4]==0) return theIdx-4;
	  if (row<3)
	    if (aField[theIdx+4]==0) return theIdx+4;
	  if (col>0){
	    if (aField[theIdx-1]==0) return theIdx-1;
	  }
	  if (col<3){
	    if (aField[theIdx+1]==0) return theIdx+1;
	  }
	  return -1;
	};
	
	// Checking for Victory !
	private boolean checkWin() {
		return Arrays.equals(aField, theField);
	}

	// Update the status text.
	private void updateStatus() {
		TextView txt = (TextView) findViewById(R.id.textStatus);
	    txt.setText(Html.fromHtml(getString(R.string.txt_status, playerTime, playerClicks)));
	}
	
	// Congratulations - you win !
	private void youWin() {
	    // Go back to the splash screen.
		finish();	
	}
	
	// Clicking a button
	private void btnClick(int row, int col){
	  int theIdx = getIndex(row, col);
	  if (aField[theIdx]!=0) {
	    int empIdx = idxEmpty(row, col);
	    if (empIdx!=-1){
	    	// Move the cell being clicked to the empty place.
	    	int id = getResources().getIdentifier("btn"+empIdx, "id", getPackageName());
		    ImageButton btn = (ImageButton) findViewById(id);
		    id = getResources().getIdentifier("cell"+ aField[theIdx], "drawable", getPackageName());
			btn.setImageResource(id);
			// Make the cell being clicked empty
			id = getResources().getIdentifier("btn"+theIdx, "id", getPackageName());
		    btn = (ImageButton) findViewById(id);
		    id = getResources().getIdentifier("cell0", "drawable", getPackageName());
			btn.setImageResource(id);
			// Do the same with the puzzle field matrix.
			aField[empIdx] = aField[theIdx];
			aField[theIdx] = 0;
	    }    
	  }
	};
	
    /* Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        // Fill the solved puzzle field
        for (int z=0; z<15; z++) {
        	theField[z] = z+1;
        }
        theField[15] = 0;
        // Add click listener to all the buttons
        for (int z=0; z<=15; z++){
			int id = getResources().getIdentifier("btn"+z, "id", getPackageName());
			ImageButton btn = (ImageButton) findViewById(id);
			btn.setOnClickListener(new View.OnClickListener() {
				
				@Override
				public void onClick(View v) {
					// Call the method with row and col parameters. 
					// It seems that this ugly way is the only way possible, since you can't get the ID as string 
					switch (v.getId()) {
					case R.id.btn0: btnClick(0, 0); break;
					case R.id.btn1: btnClick(0, 1); break;
					case R.id.btn2: btnClick(0, 2); break;
					case R.id.btn3: btnClick(0, 3); break;
					case R.id.btn4: btnClick(1, 0); break;
					case R.id.btn5: btnClick(1, 1); break;
					case R.id.btn6: btnClick(1, 2); break;
					case R.id.btn7: btnClick(1, 3); break;
					case R.id.btn8: btnClick(2, 0); break;
					case R.id.btn9: btnClick(2, 1); break;
					case R.id.btn10: btnClick(2, 2); break;
					case R.id.btn11: btnClick(2, 3); break;
					case R.id.btn12: btnClick(3, 0); break;
					case R.id.btn13: btnClick(3, 1); break;
					case R.id.btn14: btnClick(3, 2); break;
					case R.id.btn15: btnClick(3, 3); break;
					}
					// Update the status text.
					playerClicks++;
			        updateStatus();
					// Check if the puzzle is solved
			      	if (checkWin()) {
			      		youWin();
			      	}
				} // OnClick
			});
		}
        // Add a timer
        timer = new Timer();
    }
    

	@Override
    public void onStart() {
		super.onStart();
        makeField(); 
        loadField();
        playerTime = 0;
        playerClicks = 0;
        // Start the timer with one second interval
        timer.scheduleAtFixedRate(new TimerTask() {
        	@Override
        	public void run() {
        		onTimer();
        	}
        }, 0, 1000);
        updateStatus();
    }
	
	private void onTimer() {
		// To make changing view possible from different thread
		runOnUiThread(runTimer);
	}
	
	private Runnable runTimer = new Runnable () {
		@Override
		public void run() {
			playerTime++;
			updateStatus();
		}
	};
}